/************************* MPEG-2 NBC Audio Decoder **************************
 *                                                                           *
"This software module was originally developed by 
AT&T, Dolby Laboratories, Fraunhofer Gesellschaft IIS 
and edited by
Yoshiaki Oikawa (Sony Corporation),
Mitsuyuki Hatanaka (Sony Corporation)
Yasuhiro Toguri (Sony Corporation)
in the course of development of the MPEG-2 NBC/MPEG-4 Audio standard ISO/IEC 13818-7, 
14496-1,2 and 3. This software module is an implementation of a part of one or more 
MPEG-2 NBC/MPEG-4 Audio tools as specified by the MPEG-2 NBC/MPEG-4 
Audio standard. ISO/IEC  gives users of the MPEG-2 NBC/MPEG-4 Audio 
standards free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-2 NBC/MPEG-4
Audio  standards. Those intending to use this software module in hardware or 
software products are advised that this use may infringe existing patents. 
The original developer of this software module and his/her company, the subsequent 
editors and their companies, and ISO/IEC have no liability for use of this software 
module or modifications thereof in an implementation. Copyright is not released for 
non MPEG-2 NBC/MPEG-4 Audio conforming products.The original developer
retains full right to use the code for his/her  own purpose, assign or donate the 
code to a third party and to inhibit third party from using the code for non 
MPEG-2 NBC/MPEG-4 Audio conforming products. This copyright notice must
be included in all copies or derivative works." 
Copyright(c)1996.
 *                                                                           *
 ****************************************************************************/
/* April-2005 modified by Chan Ti Eu (Institute of Infocomm Research, A*STAR) for */
/* multiple SLS elementary stream support										  */


/* for AudioSpecificInfo */
#define	lenAudioObjectType		5
#define lenSamplingFrequencyIndex	4
#define lenSamplingFrequency		24
#define lenChannelConfiguration		4
#define lenFrameLengthFlag	    	1
#define	lenDependsOnCoreCoder		1
#define lenExtensionFlag		1
#define	lenASIHeader			(5+4+0+4+1+1+1)

#ifdef I2R_LOSSLESS
#define lenpcmWordLength	3
#define lenframeLength		3
#endif

#include "all.h"
#include "port.h"
#include <fcntl.h>
#include <sys/stat.h>

#ifdef WIN32
#include <io.h>
#else
#include <unistd.h>
#endif

#include "aiff_support.h"

#ifdef	DOLBY_MDCT
#include "block.h"
#include "dolby_def.h"
#endif
#include "nok_lt_prediction.h"
#ifdef USELIB_ISOMP4
#include "MP4Movies.h"
#define     MAX_BUFFER_SIZE         (1024*1024)
#endif


#define PATH_LEN    128


void
init_config(int extended_show)
{
  int objtype, samplingInfo, tmp;
  int nChannelConf;
  int lle_main_stream;
  int core_present;

  /* Audio Specific Configuration */
  objtype = getbits(lenAudioObjectType); /* from AudioSpecificConfig */

  if (extended_show) printf("Object type               %d\n",objtype);
  if(objtype > LTP_Profile + 1 || objtype < Main_Profile + 1 )
#ifdef I2R_LOSSLESS
  if (!((objtype == 29) || (objtype == 30)))
#endif
  {
    printf("Warning!!! Invalid Object Type in AudioSpecificConfig\n");
  }

  samplingInfo = getbits(lenSamplingFrequencyIndex);
  if (extended_show) printf("SamplingFrequencyIndex    %d\n",samplingInfo);

  nChannelConf = getbits(lenChannelConfiguration);
  if (extended_show) printf("ChannelConfiguration      %d\n",nChannelConf);

#ifdef I2R_LOSSLESS
  if ((objtype == 29) || (objtype == 30)) {
	  int frameLength = 0;
	  /* SLS Specific Configuration */
	   mc_info.pcmWordLength = getbits(lenpcmWordLength); 
	   core_present = getbits(1);
	   lle_main_stream = getbits(1);
	   getbits(1);  /* reserved*/
 	  frameLength = getbits(lenframeLength); 
		switch (frameLength)
		{
			case 0:
				mc_info.osf = 1;
				break;
			case 1:
				mc_info.osf = 2;
				break;
			case 2:
				mc_info.osf = 4;
				break;
		}
	  if (objtype == 30) {
		  if (!nChannelConf) {
				  get_prog_config(&prog_config);
		  }
	  }
#ifndef ADIF2MP4
	set_parameters(samp_rate_info[samplingInfo].samp_rate, mc_info.osf,core_present);
#endif
    printf("\nAAZ track(s) found. pcmWordLength = %d frameLength = %d\n", mc_info.pcmWordLength, frameLength);
  }
#else  
  /* GA Specific Configurtaion */
  tmp = getbits(lenFrameLengthFlag);
  if (extended_show) printf("FrameLengthFlag           %d\n",tmp);
  getbits(lenDependsOnCoreCoder);
  getbits(lenExtensionFlag);
#endif

  /* pce */
  if(nChannelConf==0) {
    get_prog_config(&prog_config);
    if (extended_show)  {
      PRINT( SE , "\n" );
      print_PCE(&prog_config);
    }
    mc_info.profile = prog_config.profile; 
    if (extended_show) printf("ProfileID in PCE          %d\n",mc_info.profile);
    if(mc_info.profile > SSR_Profile || mc_info.profile < Main_Profile){
      printf("Warning!!! Invalid Profile ID in PCE\n");
    }
  }
  else {
    mc_info.profile = objtype -1 ; /* Use object type for AAC Profile */
#ifdef I2R_LOSSLESS
  if (!((objtype == 29) || (objtype == 30))) 
#endif
    mc_info.sampling_rate_idx = samplingInfo;

    infoinit(&samp_rate_info[mc_info.sampling_rate_idx]);
  }
}


#if USELIB_ISOMP4
void init_mp4file(MP4Track* trak, MP4Handle* decoderSpecificInfoH
#ifdef I2R_LOSSLESS
					,u32 *streamPriority
					,u32 *sls
#endif
				  )
{
    extern MP4Movie moov;
    MP4Err err;
    MP4Media media;
    MP4Handle initialObjectDescriptorH;

    u32 handlerType;
    u32 objectTypeIndication;
    u32 streamType;
    u32 decoderBufferSize;
//	u32 track_count;

    if ((err = MP4NewHandle( 0, &initialObjectDescriptorH ))) myexit("mp4lib");
    if ((err = MP4GetMovieInitialObjectDescriptor( moov, initialObjectDescriptorH ))) myexit("mp4lib-initialOD");
#ifdef I2R_LOSSLESS
	if ((err = MP4GetMovieTrackCount(moov, sls))) myexit("mp4lib-trackCount");
#endif
    if ((err = MP4GetMovieIndTrack( moov, /*trackNumber*/ 1, trak ))) myexit("mp4lib-IndTrack");
    if ((err = MP4GetTrackMedia( *trak, &media ))) myexit("mp4lib-TrackMedia");
    if ((err = MP4GetMediaHandlerDescription( media, &handlerType, NULL ))) myexit("mp4lib-MediaDesc");
    if ((err = MP4NewHandle( 0, decoderSpecificInfoH ))) myexit("mp4lib");
    if ((err = MP4GetMediaDecoderType( media,
                                       /*sampleDescIndex*/1, 
                                       &objectTypeIndication, 
                                       &streamType, 
                                       &decoderBufferSize, 
                                       *decoderSpecificInfoH
#ifdef I2R_LOSSLESS
									   ,streamPriority		
#endif
									   ))) myexit("mp4lib-MediaDec");
    if ( handlerType != MP4AudioHandlerType ) {
      myexit("not an audio movie");
    }
    if ( objectTypeIndication != 0x40 ) {
      myexit("not an MPEG-4 movie");
    }
    if ( streamType != 0x05 ) {
      myexit("not an audio stream");
    }
    if ( decoderBufferSize > MAX_BUFFER_SIZE ) {
      myexit("required buffer size too large");
    }
}

#ifdef I2R_LOSSLESS
void ll_init_mp4file(MP4Track* trak, MP4Handle* decoderSpecificInfoH, int i, u32 *streamPriority)
{
    extern MP4Movie moov;
    MP4Err err;
    MP4Media media;
    MP4Handle initialObjectDescriptorH;

    u32 handlerType;
    u32 objectTypeIndication;
    u32 streamType;
    u32 decoderBufferSize;
//	u32 streamPriority;

    if ((err = MP4NewHandle( 0, &initialObjectDescriptorH ))) myexit("mp4lib");
    if ((err = MP4GetMovieInitialObjectDescriptor( moov, initialObjectDescriptorH ))) myexit("mp4lib-initialOD");
    if ((err = MP4GetMovieIndTrack( moov, /*trackNumber*/ /*AAZ Track*/ i+2, trak ))) myexit("mp4lib-IndTrack");
    if ((err = MP4GetTrackMedia( *trak, &media ))) myexit("mp4lib-TrackMedia");
    if ((err = MP4GetMediaHandlerDescription( media, &handlerType, NULL ))) myexit("mp4lib-MediaDesc");
    if ((err = MP4NewHandle( 0, decoderSpecificInfoH ))) myexit("mp4lib");
    if ((err = MP4GetMediaDecoderType( media,
                                       /*sampleDescIndex*/1, 
                                       &objectTypeIndication, 
                                       &streamType, 
                                       &decoderBufferSize, 
                                       *decoderSpecificInfoH
#ifdef I2R_LOSSLESS
										,streamPriority
#endif
									   ))) myexit("mp4lib-MediaDec");
    if ( handlerType != MP4AudioHandlerType ) {
      myexit("not an audio movie");
    }
    if ( objectTypeIndication != 0x40 ) {
      myexit("not an MPEG-4 movie");
    }
    if ( streamType != 0x05 ) {
      myexit("not an audio stream");
    }
    if ( decoderBufferSize > MAX_BUFFER_SIZE ) {
      myexit("required buffer size too large");
    }
}
#endif
#endif
