/*****************************************************************************
 *                                                                           *
SC 29 Software Copyright Licencing Disclaimer:

This software module was originally developed by
  AT&T, Dolby Laboratories, Fraunhofer IIS-A

and edited by
  Yoshiaki Oikawa (Sony Corporation),
  Mitsuyuki Hatanaka (Sony Corporation)

in the course of development of the ISO/IEC 13818-7 and ISO/IEC 14496-3 
standards for reference purposes and its performance may not have been 
optimized. This software module is an implementation of one or more tools as 
specified by the ISO/IEC 13818-7 and ISO/IEC 14496-3 standards.
ISO/IEC gives users free license to this software module or modifications 
thereof for use in products claiming conformance to audiovisual and 
image-coding related ITU Recommendations and/or ISO/IEC International 
Standards. ISO/IEC gives users the same free license to this software module or 
modifications thereof for research purposes and further ISO/IEC standardisation.
Those intending to use this software module in products are advised that its 
use may infringe existing patents. ISO/IEC have no liability for use of this 
software module or modifications thereof. Copyright is not released for 
products that do not conform to audiovisual and image-coding related ITU 
Recommendations and/or ISO/IEC International Standards.
The original developer retains full right to modify and use the code for its 
own purpose, assign or donate the code to a third party and to inhibit third 
parties from using the code for products that do not conform to audiovisual and 
image-coding related ITU Recommendations and/or ISO/IEC International Standards.
This copyright notice must be included in all copies or derivative works.
Copyright (c) ISO/IEC 1996.
 *                                                                           *
 ****************************************************************************/

#include "config.h"
#include "ct_sbrdecoder.h"

#include "adifdec.h"
#include "diagnose.h"
#include "audioout.h"
#include "bitinput.h"
#include "config.h"
#include "common.h"
#include "ct_sbrdecoderConfTool.h"
#include "ct_sbrConfToolDownSampler.h"
#include "audiofile_io.h"

#ifdef DRC
#include "drc.h" /* to control drc settings */
#endif
#include "gc.h"  /* to contol ssr settings */

#ifdef PARAMETRICSTEREO
int doPsConformance = 0;
#endif

static void             usage(char *s);
static int              parse_cmdline(int argc, char *argv[]);
static void             print_banner(char *cmd);

/* parameters */
extern int delay;
extern int fileIndex;

static char *bitstreaminputfile;

long            bno;
int             debug[256];
int             delay;
char            result[256];
char            resultQmf[256];
char            *audioinput_path;

static void
print_banner(char *cmd)
{
  PRINT (SE, "\n");
  PRINT (SE, "ISO/IEC 14496-4 MPEG-4 Conformance Verification\n");
  PRINT (SE, "%s  version %s\n", cmd, SBRCONF_VERSION);


#ifdef LOW_POWER_SBR
  PRINT (SE, "LOW POWER SBR\n");
#elif PARAMETRICSTEREO
  PRINT (SE, "HIGH QUALITY SBR + PARAMETRIC STEREO\n");
#else
  PRINT (SE, "HIGH QUALITY SBR\n");
#endif
  PRINT (SE, "\n");
}

static void usage(char *s)
{
  PRINT(SE, "usage: %s [options] <bitstream> <inputfile under test>\n", s);
  PRINT(SE, " options are:\n\t");
  PRINT(SE, " -h show this help\n\t");
  PRINT(SE, " -d delay (delay introduced by decoder under test; default: 0)\n\t");
#ifdef CT_SBR
  PRINT(SE, " -l level  (Level of the HE-AAC Profile decoder (2,3,4,5), 5 is default)\n\t");
  PRINT(SE, " -r (Forces the decoder to work in down sampled SBR mode, default is off)\n\t");
#ifdef PARAMETRICSTEREO
  PRINT(SE, " -c (Enables parametric stereo decoding for HE-AAC)\n");
#endif
#endif
  myexit(0);
}

static int
parse_cmdline(int argc, char *argv[])
{
  char *cmd, *s;

  MC_Info *mip = &mc_info;

#ifdef	DRC
  Float param_drcHigh=0;
  Float param_drcLow=0;
  int param_drcTargetRefLevel=-1;
#endif
  int param_selectedProgram = -1;
  int param_ssrDecoderBand = -1;

  /* save cmd 
   * ARGBEGIN is so clever that it throws away argv[0] and 
   * increments the argv pointer
   */
  cmd = argv[0];
    
  print_banner(cmd);

  ARGBEGIN {

  default:
    PRINT(SE, "unknown option\n");
  case 'h':
    usage(cmd);
  case 'd':
    s = ARGF();
    delay = atoi(s);
    break;
#ifdef CT_SBR
  case 'l':
    s = ARGF();
    mip->HE_AAC_level = atoi(s);
    break;
  case 'r':
    mip->bDownSampledSbr = 1;
    break;
#ifdef PARAMETRICSTEREO
  case 'c':
#ifdef LOW_POWER_SBR
    PRINT(SE, "PS not available for LP-SBR\n");
    myexit("sbrEnablePS");
#endif
    mip->sbrEnablePS = 1;
    break;
#endif
#endif
  case 'D':
    for (s = ARGF(); *s; s++) {
      debug[(int)(*s)] = 1;
      PRINT(SE, "debug option %c enabled\n", *s);
    }
  }
  ARGEND
      
    if(argc != 2) {
      usage(cmd);
    }

  printf("%s %s %s\n", GREP_STRING, argv[0], argv[1]);

  /* set decoder-global variables */
  decoder_selectProgram(param_selectedProgram);
  if (param_ssrDecoderBand>0) ssr_decoder_band = param_ssrDecoderBand;
  decoder_setupParameters(mc_info.sampling_rate_idx);
#ifdef DRC
  decoder_setDrc(param_drcHigh, param_drcLow, param_drcTargetRefLevel);
#endif


  bitstreaminputfile = argv[0];

  audioinput_path = argv[1];

  if(strstr(argv[0],"sbr_qmf_")){
    fileIndex = 0;
  }
  else if(strstr(argv[0],"sbr_e_")){
    fileIndex = 1;
  }
  else if(strstr(argv[0],"sbr_gh_")){
    fileIndex = 2;
  }
  else if(strstr(argv[0],"sbr_i_")){
    fileIndex = 3;
  }
  else if(strstr(argv[0],"sbr_s_")){
    fileIndex = 4;
  }
  else if(strstr(argv[0],"sbr_twi_")){
    fileIndex = 5;
  }
#ifdef PARAMETRICSTEREO
  else if(strstr(argv[0],"sbr_ps_00")){
    fileIndex = 6;
  }
  else if(strstr(argv[0],"sbr_ps_01")){
    fileIndex = 7;
  }
#ifndef NO_PS_CONF_DCOR2
  /* these conformance criteria have been proposed in 14496-4:2004/Amd.11:2006/DCOR 2
   * balloting for this hasn't closed at the time of this writing, so they are not official.
   * Hence they can be removed from the build by defining NO_PS_CONF_DCOR2 in this file and in decoderConfTool.c
   */
  else if(strstr(argv[0],"sbr_ps_02")){
    fileIndex = 8;
  }
  else if(strstr(argv[0],"sbr_ps_03")){
    fileIndex = 9;
  }
  else if(strstr(argv[0],"sbr_ps_04")){
    fileIndex = 10;
  }
  else if(strstr(argv[0],"sbr_ps_05")){
    fileIndex = 11;
  }
  else if(strstr(argv[0],"sbr_ps_06")){
    fileIndex = 12;
  }
#endif
#endif

  return argc;
}

/* global, so that it can be closed in myexit() */
static HANDLE_BITINPUT bitstreaminput;
int
main(int argc, char *argv[])
{
  long avgBitrate = 0;
  long maxBitrate = 0;
  int bufferFullness = -1;
  int channelConfig;
  int remainingArgs;
  BUFFER_SIMULATION sim;
  int i;

  SBRDECODER sbrDec;
  AFILE  *stream1;
  audioheader audio_header1;
  int samples_read1;
  int sample_counter = 0;
  int firstHeader = 1;
  int storeIndex = 0;
  int numSamplesStored = 0;

  Float *storedData[Chans];
  float pcm_buffer1[Chans*2048];
  Float   *tmpDataReadFromFileBuffer[Chans];
  Float   *tmpDataRef[Chans];
  Float   *tmpData[Chans];
  Float   *tmpData2[Chans];
  Float   *tempDataDS[Chans];

  for (i=0;i<Chans;i++) {
    tmpData[i] = (Float *)malloc(LN*sizeof(Float));
    tmpData2[i] = (Float *)malloc(LN*sizeof(Float));
    tempDataDS[i] = (Float *)malloc(LN*sizeof(Float));
    tmpDataRef[i] = (Float *)malloc(LN*sizeof(Float));
    tmpDataReadFromFileBuffer[i] = (Float *)malloc(2*LN*sizeof(Float));
    /* kaehleof 20050217: yuck... who has had this idea? */
    storedData[i] = (Float *)malloc(3000000*sizeof(*storedData[0]));
  }

  decoder_init();
  remainingArgs = parse_cmdline(argc, argv);	    /* parse command line */

  bitstreaminput = bitinput_openFile(bitstreaminputfile,
                                     &prog_config,
                                     &mc_info,
                                     &channelConfig,
                                     &maxBitrate,
                                     &avgBitrate,
                                     &bufferFullness,
                                     (remainingArgs==1));

  decoder_setSBR_conformance(1);
  decoder_setupParameters(mc_info.sampling_rate_idx);
  if (bitstreaminput==NULL) myexit("error opening bitstream");
  if (remainingArgs==1) myexit(NULL);

  /* prepare buffer simulation */
  sim = create_bitBuffer_sim();
  {
    long bufferSize;
    double avgFrameLen = .0;

    getDecoderBufferSize(&prog_config, channelConfig, &bufferSize);
    if (NULL != bitinput_getCurrentADTSHeader(bitstreaminput)) avgBitrate = 0; /* get more accurate estimate from ADTS-Headers */

    if (avgBitrate>0) avgFrameLen = BITRATE2FRAMELEN(avgBitrate);
    else if (avgBitrate==0) avgFrameLen = 0; /* unknown, calculate on the fly */
    else if (avgBitrate<0) avgFrameLen = -1; /* variable bitrate */

    if (bufferFullness == 0x7fffffff) { /* also indicates variable bitrate */
      bufferFullness = 0;
      avgFrameLen = -1;
    }

    bitBuffer_sim_setFrameLen(sim, avgFrameLen, bufferSize);
    if (maxBitrate>0) bitBuffer_sim_testPeakBitrate(sim, maxBitrate);
  }



  /* main loop */
  for(bno = 0;; bno++) {
    SBRBITSTREAM *sbrBitStr = NULL;
    Float **output_data;
    int output_blockLen;
    int err = bitinput_getNextBlock(bitstreaminput);
    if (err == -2) break; /* eof */
    if (err) myexit("error reading next block");

    dec(&output_data, &output_blockLen);

    /* look for SBR data */
    if (mc_info.sbrPresentFlag != 0) {
      if ((sbrBitStr = (SBRBITSTREAM*)decoder_getSBRbitstream())!=NULL)
        if (sbrBitStr->NrElements == 0) sbrBitStr=NULL;
    }

    /* do SBR conformance */
    if (sbrBitStr!=NULL) {
      if (bno == 0) {

        Twiddle twiddle, *ptrTwiddle=NULL;
        FILE *twiddlefile;
#if !defined LOW_POWER_SBR
        if( (twiddlefile = fopen("twiddle.dat", "rb")) ) {
          double fi, beta;
          unsigned int count;
          count  = fread(&fi,   1, sizeof(double), twiddlefile);
          count += fread(&beta, 1, sizeof(double), twiddlefile);
          if(mc_info.bDownSampledSbr) {
            count += fread(&fi,   1, sizeof(double), twiddlefile);
            count += fread(&beta, 1, sizeof(double), twiddlefile);
          }
          if(count == 2 * sizeof(double) * (mc_info.bDownSampledSbr + 1)) {
            ptrTwiddle = &twiddle;
            ptrTwiddle->fi   = fi;
            ptrTwiddle->beta = beta;
            PRINT(SE, "using fi = %f, beta = %f\n", fi, beta);
          }
          else {
            if (fileIndex != 5) {
              PRINT(SE, "warning: ignoring short twiddle-file\n");
            }
          }
          fclose(twiddlefile);
        }
#endif

        sbrDec = openSBR(mc_info.sampling_rate, mc_info.bDownSampledSbr, ptrTwiddle);

        if (sbrDec==NULL) {
          myexit("openSBR failed!\n");
        }

#if !defined LOW_POWER_SBR
        if (fileIndex == 5) {
          FILE *twiddlefile = NULL;
          int headerAvailable = 0;
          SBR_ERROR err = SBRDEC_OK ;

          open_inputaudio (&stream1,
                           audioinput_path,
                           &audio_header1);

          if (audio_header1.samplefreq != mc_info.sampling_rate * (2 - mc_info.bDownSampledSbr)) {
            myexit("sampling rates of input files differ!\n");
          }

          samples_read1 = get_audiosample ( (float*) storedData[0],
                                            stream1,
                                            audio_header1.channels,
                                            0,
                                            16384 * (2 - mc_info.bDownSampledSbr));

          close_inputaudio (stream1);

          err = readAndDecodeSBRConfTool(sbrDec,
                                         sbrBitStr,
                                         mc_info.bDownSampledSbr,
                                         mc_info.ch_info,
                                         &headerAvailable);

          twiddle = identifyQmfBank( (float*) storedData[0],
                                     audio_header1.channels,
                                     mc_info.bDownSampledSbr,
                                     getSBRstartBand (),
                                     getSBRstopBand  () );

          closeSBR (sbrDec);

          sbrDec = openSBR(mc_info.sampling_rate,
                           mc_info.bDownSampledSbr,
                           &twiddle);

          if (sbrDec == NULL) {
            myexit("openSBR failed!\n");
          }

          if ((twiddlefile = fopen("twiddle.dat", "rb+")) || (twiddlefile = fopen("twiddle.dat", "wb+"))) {
            if (mc_info.bDownSampledSbr) {
              fseek(twiddlefile, 2 * sizeof(double), SEEK_SET);
            }
            fwrite(&twiddle.fi,   1, sizeof(double), twiddlefile);
            fwrite(&twiddle.beta, 1, sizeof(double), twiddlefile);
            fclose(twiddlefile);
          } else {
            PRINT(SE, "couldn't open or create twiddle.dat. Maybe permission denied?");
          }
        }
#endif

        open_inputaudio (&stream1,
                         audioinput_path,
                         &audio_header1);
        sample_counter = delay;

        if (audio_header1.samplefreq != mc_info.sampling_rate * (2 - mc_info.bDownSampledSbr)) {
          myexit("sampling rates of input files differ!\n");
        }

#ifdef PARAMETRICSTEREO
        /* we should compare both channels if
           - the stream has only one channel (sce)
           - the inputfile has two channels
           - the -c switch was given
        */
        {
          int n_channels = 0;
          for (i=0; i < Chans; i++) {
            if (mc_info.ch_info[i].present && mc_info.ch_info[i].cpe == 0) {
              n_channels++;
            }
          }
          if (n_channels == 1 && audio_header1.channels == 2 && mc_info.sbrEnablePS) {
            doPsConformance = 1;
          }
        }
#endif
      }

      samples_read1 = get_audiosample (pcm_buffer1,
                                       stream1,
                                       audio_header1.channels,
                                       sample_counter,
                                       (1024*(2 - mc_info.bDownSampledSbr)));
      if (samples_read1==0) {
        /* end of input file -> wait delay more samples */
        if (delay > 0) {
          int i;
          delay -= 1024*(2 - mc_info.bDownSampledSbr);
          for (i = 0; i < Chans * 2048; i++)
            pcm_buffer1[i] = 0;
        }
        else {
          myexit("Warning! Short inputfile\n");
        }
      }
      sample_counter += (1024*(2 - mc_info.bDownSampledSbr));

      /* Copy to tmpData*/
      {
        int channel = 0;
        for (i=0; i<Chans; i++) {
          int kk;
          if (!(mc_info.ch_info[i].present))
            continue;

          for(kk = 0; kk < (1024*(2 - mc_info.bDownSampledSbr)); kk++){
            tmpDataReadFromFileBuffer [i][kk] = tmpDataReadFromFileBuffer [i][kk+(1024*(2 - mc_info.bDownSampledSbr))];
            tmpData[i][kk] = pcm_buffer1 [(kk * audio_header1.channels) + channel];
            tmpDataReadFromFileBuffer [i][kk+(1024*(2 - mc_info.bDownSampledSbr))] = tmpData[i][kk];
          }
          channel++;
#ifdef PARAMETRICSTEREO
          if (doPsConformance) {
            if (i >= Chans - 1) myexit("Oh no! Ps was in the last channel! That won't work! Giving up\n");
            for(kk = 0; kk < (1024*(2 - mc_info.bDownSampledSbr)); kk++){
              tmpDataReadFromFileBuffer [i + 1][kk] = tmpDataReadFromFileBuffer [i + 1][kk+(1024*(2 - mc_info.bDownSampledSbr))];
              tmpData[i + 1][kk] = pcm_buffer1 [(kk * audio_header1.channels) + channel];
              tmpDataReadFromFileBuffer [i + 1][kk+(1024*(2 - mc_info.bDownSampledSbr))] = tmpData[i + 1][kk];
            }
            /* no more channels -> go on */
            break;
          }
#endif
        }
      }

      for (i=0; i<Chans; i++) {
        int kk;
        if (!(mc_info.ch_info[i].present))
          continue;
        for (kk=0; kk<LN2; kk++)
          tempDataDS[i][kk] = output_data[i][kk];
      }

      if (fileIndex == 0 || fileIndex == 5) {
        int frameSize = (mc_info.bDownSampledSbr ? 1024:2048);
        /* qmf-test is different */
        if (applySBR(sbrDec,
                     sbrBitStr,
                     output_data,
                     tmpDataRef,
                     mc_info.bDownSampledSbr,
#ifdef PARAMETRICSTEREO
                     mc_info.sbrEnablePS,
#endif
                     mc_info.ch_info) != SBRDEC_OK)
          myexit("invalid sbr bitstream");
        calculateDiffandRms(tmpData, tmpDataRef, mc_info.ch_info,frameSize);
      } else {
        int headerAvailable = 0;
        SBR_ERROR err = SBRDEC_OK ;

        err = readAndDecodeSBRConfTool(sbrDec,
                                       sbrBitStr,
                                       mc_info.bDownSampledSbr,
                                       mc_info.ch_info,
                                       &headerAvailable);

        /* if header available use stored data*/
        if(headerAvailable){
          if(firstHeader){
            firstHeader = 0;
            numSamplesStored = storeIndex;
            storeIndex = 0;
          }

          err = applySBRConfTool(sbrDec,
                                 sbrBitStr,
                                 storedData,
                                 tmpDataRef,
                                 mc_info.bDownSampledSbr,
                                 mc_info.ch_info,
#ifdef PARAMETRICSTEREO
                                 mc_info.sbrEnablePS,
#endif
                                 storeIndex);

          if(storeIndex > 2*1024 && storeIndex < numSamplesStored - 2*1024){
            int frameSize = (mc_info.bDownSampledSbr ? 1024:2048);
            calculateDiffandRms(tmpData, tmpDataRef, mc_info.ch_info, frameSize);
          }

          storeIndex += 1024;

        } else {
          /* Down sample signal*/

          {
            int i;

            for (i=0; i<Chans; i++) {
              if (!(mc_info.ch_info[i].present))
                continue;
              downSampleSbr(tmpData, (mc_info.upsamplingFactor == 2.0f), i);
              memcpy(storedData[i] + storeIndex,tmpData[i],1024*sizeof(double));
            }

            mc_info.upsamplingFactor = 1;
          }

          /* Let's upsample the signal again, to see how much it differs, from the one we just had.*/
          if(storeIndex > 2*1024){
            int frameSize = (mc_info.bDownSampledSbr ? 1024:2048);

            err = applySBRConfTool(sbrDec,
                                   sbrBitStr,
                                   storedData,
                                   tmpData2,
                                   mc_info.bDownSampledSbr,
                                   mc_info.ch_info,
#ifdef PARAMETRICSTEREO
                                   mc_info.sbrEnablePS,
#endif
                                   (storeIndex-1024));

            calculateDiffandRmsQmf(tmpData2, tmpDataReadFromFileBuffer, mc_info.ch_info,frameSize);
          }

          storeIndex += 1024;
         }
      }
    }


    /* do buffer simulation */
    {
      const ADTS_Header *tmp_head = bitinput_getCurrentADTSHeader(bitstreaminput);
      update_bitBuffer_sim(sim, bitinput_resetCounter(bitstreaminput)<<3, tmp_head);
    }
    /* initial buffer fullness */
    if ((bno==0)&&(bufferFullness!=-1)) {
      bitBuffer_sim_adjustFullness(sim, bufferFullness);
    }
    check_bitBuffer_sim(sim, "", debug['b'], debug['B'], debug['R']);
  }

  myexit(NULL);
  return (EXIT_SUCCESS);
}


void
myexit(char *s)
{
  bitinput_close(bitstreaminput);
  decoder_free();
  printf("%s\n", resultQmf);
  printf("%s\n", result);
  if (s) PRINT(SE,"\n%s\n", s);
  exit(s ? 1 : 0);
}
