#!/usr/bin/perl
## START_OF_HELP ##
#extract used tools from bitstream (using mp4mcDec or any other de-/transcoder)
#also check buffer requirements
#and some more stuff
#
#commandline parameters:
#   not using a decoder:
#     -log <file>        process logfile instead of calling any program
#   using a decoder:
#     -dec <cmdline>     de-/transcode using the given commandline
#                        (dflt:<kaehleof compiled refsoft>)
#     -decif <text>      prefix in front of inputfile
#     -decof <text>      prefix in front of outputfile
#     -ifof / -ofif      give in- or output file first on the decoder commandline
#                        (dflt:-ifof)
#     -suffix <text>     suffix added to the output file, - to keep from input
#                        (dflt:wav)
#   controlling the output:
#     -line / -long      return grep-friendly line or a human readable long form
#                        (dflt:-long)
#     -compare <file>    look for information on tested filename in <file> and
#                        print result of comparison
#
## END_OF_HELP ##
#
# very powerful in combination with mp4files.nfo and a bash-line like:
# for x in *.mp4;do fn=${x%_*};fn=${fn##*/};str="$fn: "`checkmp4_mcdec.pl $x`;str1=`grep -i "^ *$fn:" mp4files.nfo`;str1=$fn${str1##*$fn};if echo "$str" | grep -iqxE "$str1";then echo "$fn succeded";else echo "$fn failed ($x)"|sed -e's@(.*/@(@';echo "should be:$str1";echo "  but was:$str";fi;done 2>&1|tee filetests.log
#
# ...in english:
# for all *.mp4 files:
#   find the basename (al00, ac01,...) $fn by removing everything behind the
#   first _ and everything before the last /
#   run checkmp4.pl
#   build a new line $str, which is like the lines in mp4files.nfo
#   now grep for the line starting with $fn in the mp4files.nfo
#   now grep $str for the (regular expression) found from mp4files.nfo
#   echo "succeeding" or "failed
# ... really simple...
#   
#
# ... or in case that is too much ASCII-Trash for you: the same can be done with:
# for x in *.mp4;do checkmp4.pl -compare mp4files.nfo;done


sub cat_as_help {
  local $file;
  local $cat=0;
  if (!($file = shift)) { $file = $0; }

  open(HELPFILE, "< $file");

  while ($x = <HELPFILE>) {
    if ($cat == 0) {
      if ($x =~ /\#.*START_OF_HELP.*\#/) {
        $cat = 1;
      }
    } elsif ($cat == 1) {
      if ($x =~ /\#.*END_OF_HELP.*\#/) {
        $cat = 2;last;
      } else {
        $x =~ s/^\#*//;
        print $x;
      }
    }
  }

  close(HELPFILE);

  if ($cat==0) { return -1; }
  return 0;
}


$arg_logfile = "";
$arg_filename = "";

$basepath = "..";

#   check with rewrite
$arg_reverseinout = 1;
$arg_cmdline = "$basepath/bin/mp4AudVm_Tbt_VmMp4AuDecRewrite/Linux_v2_c1_eptool_sbr_gcc_O3/mp4audec -daac VbB -d 9 -o";
$arg_ifprefix = "";
$arg_ofprefix = "";

#   check with mcDec
#$arg_reverseinout = 0;
#$arg_cmdline = "$basepath/bin/mp4mcDec/Linux_isomp4_sbr_AFsp_gcc_O3/mp4audec_mc -DChVbt";
#$arg_ifprefix = "";
#$arg_ofprefix = "";

$arg_outsuffix = "wav";


$arg_outputmode = "long";
$arg_comparefile = "";
$arg_comparefs = 0;


while ($arg = shift @ARGV) {
#  print "argv got $arg\n";
  if ($arg =~ /^-h$/i) {
    if (cat_as_help($0)) { print "no help available"; }
    exit;
#not using a decoder
  } elsif ($arg =~ /^-log$/i) {
    $arg_logfile = shift @ARGV;
#using a decoder
  } elsif ($arg =~ /^-dec$/i) {
    $arg_cmdline = shift @ARGV;
  } elsif ($arg =~ /^-decif$/i) {
    $arg_ifprefix = shift @ARGV;
  } elsif ($arg =~ /^-decof$/i) {
    $arg_ofprefix = shift @ARGV;
  } elsif ($arg =~ /^-suffix$/i) {
    $arg_outsuffix = shift @ARGV;
  } elsif ($arg =~ /^-ifof$/i) {
    $arg_reverseinout = 0;
  } elsif ($arg =~ /^-ofif$/i) {
    $arg_reverseinout = 1;
#controlling the output
  } elsif ($arg =~ /^-line$/i) {
    $arg_outputmode = "line";
  } elsif ($arg =~ /^-long$/i) {
    $arg_outputmode = "long";
  } elsif ($arg =~ /^-comparefs$/i) {
    $arg_comparefs = 1;
  } elsif ($arg =~ /^-compare$/i) {
    $arg_comparefile = shift @ARGV;
    $arg_outputmode = "compare";
#...and last but not least the filename
  } else {
    $arg_filename = $arg;
  }
}


##############


if ($arg_logfile) {
  ### Get info from logfile
  if (! -f $arg_logfile ) { die "error opening file $arg_logfile" }
  open(LOG, "<$arg_logfile");
} else {
  ### Get info from decoder
  local $fileEnding;
  local $tmpfilename = "out";

  if (! -f $arg_filename ) { die "error opening file $arg_filename" }

  if ($arg_outsuffix=~/^-$/) {
    $fileEnding = $arg_filename;
    $fileEnding =~s/.*\.//;
  } else {
    $fileEnding = $arg_outsuffix;
  }

  $tmpfilename = $tmpfilename.".".$fileEnding;

  if ( $arg_reverseinout==1 ) {
    open(LOG,"$arg_cmdline $arg_ofprefix $tmpfilename $arg_ifprefix $arg_filename 2>&1 |");
  } else {
    open(LOG,"$arg_cmdline $arg_ifprefix $arg_filename $arg_ofprefix $tmpfilename 2>&1 |");
  }
}


##############


$track_total = -1;
$AOTs = "";
$ChannelConfig = -1;
$samling_rate = -1;

$torture_coeff=1241;
$torture_scale=121;
$torture_escap=8177;

$PCE_num = 0;
$PCE_parse_mode = 0;
$PCE_parse_channels = 'norm';
$PCE_profile = -1;
$CPEs=0;$SCEs=0;$LFEs=0;
$cupIND=0;$cupDEP=0;


### error/tools collection

$errors = "";
$toolsused = "";
$specials = "";

sub add_to_list {
  my $lst = shift;
  my $msg = shift;
  my $pat = quotemeta($msg);
  if (length($lst)==0) { $lst = "$msg"; }
  elsif ($lst !~ /$pat/) { $lst = $lst." ".$msg; }
  return $lst;
}

while (<LOG>) {
  $x = $_;
  #print STDERR "got $x\n";
  #if ($x =~ /block(?: *)(\d*)/i) {print "block $1";}

  ## get errors and warnings
  if ($x =~ /dependsOnCoreCoder-flag incorrect/) 
  {
    $errors = add_to_list($errors,"dependsOnCore");
  } 
  elsif ($x =~ /layerNr-field incorrect/) 
  {
    $errors = add_to_list($errors,"layerNr");
  } 
  elsif ($x =~ /directMapping-flag incorrect/) {
    $errors = add_to_list($errors,"directMapping");
  } elsif ($x =~ /decoderBufferSize incorrect/) 
  { $errors = add_to_list($errors,"decoderBufferSize");
  } elsif ($x =~ /IOD does not contain ES descriptors/) {
    $errors = add_to_list($errors,"incomplete-IOD");
  } elsif ($x =~ /Diagnosis found a total of \d+ warning/) {
  } elsif ($x =~ /no standards.*channels in.* file.*multiple/i) {
  } elsif ($x =~ /max_sfb decreasing/) {
    $errors = add_to_list($errors,"max_sfb");
  } elsif ($x =~ /(?:bitBuffer-simulation: |bitBuffer.*: )WARNING/i) {
    $errors = add_to_list($errors, "Buffer-Warning");
  } elsif ($x =~ /(?:bitBuffer-simulation: |bitBuffer.*: )ERROR/i) {
    $errors = add_to_list($errors, "Buffer-Error");
  ## temporary elsif for quota-overflow protection ;)
  } elsif ($x =~ /BsGetBit: error reading bit stream|BsReadByte: not enough bits left in stream/)
  {}
  ## ...
  elsif ($x =~ /incorrect|error(?! resilient)|warning/i) { ### last resort...
    print "unknown error: $x";
    $errors = add_to_list($errors,"unknown-more");
  }

  ## get tools used (AAC)
  if ($x =~ /\#(.*)detected/) {
    foreach $y ( split ' ',$1 ) {
      $y =~ s/^WndShape$/WndShapeSW/i;
      $toolsused = add_to_list($toolsused, $y);
    }
  }

  ## get specials, similar to warnings
  if ($x =~ /Non meaningfull? window[_ ]sequence transis?tion/) { $specials = add_to_list($specials, "nonmeaningful_ws_transition"); }
  if ($x =~ /Buffer-Test detected/) { $specials = add_to_list($specials, "BufferTest"); }
  if ($x =~ /ADTS-header id has to be 1 for MPEG2-AAC, is 0/) { $specials = add_to_list($specials, "MPEG4"); }
  if ($x =~ /CRC.*OK/i) { $specials = add_to_list($specials, "CRC"); }
  if ($x =~ /CRC.*FAIL/i) { $errors = add_to_list($errors, "CRC-Error"); }

  ## stuff from Object Descriptor
  if ($x =~ /Found.*with *(\d*) *suitable tracks/i) {
    $track_total = $1;
  }
  if ($x =~ /(?:Object type|audioSpC->audioObjType)(?:[: ]*)(.*)/) {
    my $tmp = $1; $tmp =~ s/ /_/g;
    $AOTs = add_to_list($AOTs, $tmp);
  }
  if ($x =~ /ChannelConfig(?:uration)*(?: *:? *)(\d)/i) {
    $ChannelConfig=$1;
    if ($ChannelConfig =~ /1/) {$SCEs=1;}
    elsif ($ChannelConfig =~ /2/) {$CPEs=1;}
    elsif ($ChannelConfig =~ /3/) {$SCEs=1;$CPEs=1;}
    elsif ($ChannelConfig =~ /4/) {$SCEs=2;$CPEs=1;}
    elsif ($ChannelConfig =~ /5/) {$SCEs=1;$CPEs=2;}
    elsif ($ChannelConfig =~ /6/) {$SCEs=1;$CPEs=2;$LFEs=1;}
    elsif ($ChannelConfig =~ /7/) {$SCEs=1;$CPEs=3;$LFEs=1;}
  }

  ## for arithmetic torture detection
  if ($x =~ /unused coefficients: (\d*)/) {$torture_coeff=$1;}
  if ($x =~ /unused scalefactors: (\d*)/) {$torture_scale=$1;}
  if ($x =~ /unused escapes: (\d*)/) {$torture_escap=$1;}

  ## sampling rate
  if ($x =~ /(?:p->sampling_rate|audioSpC->samplingFrequency)(?:[: ]*)(.*)/) {
    $sampling_rate = $1;
    $sampling_rate =~ s/\d\d\d$//;
    $sampling_rate =~ s/^(\d)$/0$1/;
  }
  if ($x =~ /(?:p->sampling_rate_idx)(?:[: ]*)(.*)/) {
    $sampling_rate=$1;
    $sampling_rate =~ s/^12$/07/;
    $sampling_rate =~ s/^11$/08/;
    $sampling_rate =~ s/^10$/11/;
    $sampling_rate =~ s/^9$/12/;
    $sampling_rate =~ s/^8$/16/;
    $sampling_rate =~ s/^7$/22/;
    $sampling_rate =~ s/^6$/24/;
    $sampling_rate =~ s/^5$/32/;
    $sampling_rate =~ s/^4$/44/;
    $sampling_rate =~ s/^3$/48/;
    $sampling_rate =~ s/^2$/64/;
    $sampling_rate =~ s/^1$/88/;
    $sampling_rate =~ s/^0$/96/;
  }

  ## ADIF/ADTS and PCEs... a wonderful mess...
  if ($x =~ /number of PCE(?: *)(.*)/) {$PCE_num = $1}

  local $got_pce_content = 0;
  if ($x =~ /p->profile(?: *)(.*)/) {
    $PCE_profile = $1;
    $got_pce_content = 1;
  } elsif ($x =~ /p->(.*)/) {
    $PCE_parse_mode = 0;
    if ( $x =~ 'p->front'|'p->side'|'p->back' ) {$PCE_parse_channels = 'norm';$got_pce_content=1;}
    if ( $x =~ 'p->lfe' ) {$PCE_parse_channels = 'lfe';$got_pce_content=1;}
    if ( $x =~ 'p->coupling' ) {$PCE_parse_channels = 'coupling';$got_pce_content=1;}
  }
  

  if ( $PCE_parse_mode==1 ) {
    $PCE_channel_lines{$PCE_parse_channels} = $PCE_channel_lines{$PCE_parse_channels}.$x;
  }
  if ( ($x =~ '   idx    cpe    tag')||($x =~ '   idx    ind    tag') ) { $PCE_parse_mode=1; $got_pce_content=1; }

  if ($got_pce_content!=0) {
    if ($PCE_num==0) { $PCE_num=1; }
  }

  ## thats all for now...
}

close(LOG);


## some processing of the data from above

# arithmetic troture
#if (($torture_coeff<124)&&($torture_scale<=121)&&($torture_escap<8192)) {
if (($torture_coeff<62)&&($torture_scale<=121)&&($torture_escap<8192)) {
  $specials = add_to_list($specials, "arithmetic_torture");
}

if ($arg_filename) {
  $filename_sr = $arg_filename;
  $filename_sr =~ s/.*_(\d\d).*/$1/;
  if ($arg_comparefs == 1) {
    if ($filename_sr!=$sampling_rate) {
      $errors = add_to_list($errors,"wrong_samp_rate_".$sampling_rate."!=".$filename_sr);
    }
  }
}

@toolsused = sort (split ' ',$toolsused);
@specials = sort (split ' ',$specials);
@errors = sort (split ' ',$errors);


# PCE parsing
if ($PCE_num!=0) {
  $CPEs=0;$SCEs=0;$LFEs=0;
  $cupIND=0;$cupDEP=0;
  
  #normal channels
  @a=sort (split('\n',$PCE_channel_lines{'norm'}));
  my $last_a = "";
  foreach $x (@a) {
    next if ($x eq $last_a); $last_a = $x;
    if ($x =~ /^( +)[0123456789]+( +)1(.+)/) { $CPEs++ }
    else { $SCEs++ }
  }

  #lfe channels
  @a=sort split('\n',$PCE_channel_lines{'lfe'});
  $last_a = "";
  foreach $x (@a) {
    next if ($x eq $last_a); $last_a = $x;
    $LFEs++;
    #if ($x =~ /^( +)[0123456789]+( +)1(.+)/) { $lfeCPEs++ }
    #else { $lfeSCEs++ }
  }
  #$totalLFEs=$lfeSCEs+$lfeCPEs*2;

  #coupling channels
  @a=sort split('\n',$PCE_channel_lines{'coupling'});
  $last_a = "";
  foreach $x (@a) {
    next if ($x eq $last_a); $last_a = $x;
    if ($x =~ /^( +)[0123456789]+( +)1(.+)/) { $cupIND++ }
    else { $cupDEP++ }
  }
}

### output gathered data

if (( $arg_outputmode =~ /line/ )||( $arg_outputmode =~ /compare/ )) {
  #### ---- LINE mode output
  local $line = "";

  foreach $x (@toolsused) {$line = $line."$x "};
  foreach $x (@specials)  {$line = $line."$x "};

  #print "number of PCEs : $numPCE\n";

  if ( length($AOTs) == /0/ ) {$AOT=$PCE_profile+1}
  else { $AOT = $AOTs }
  $line = $line."AOT: $AOT";

  if ($PCE_num!=0) {
    $line = $line.", SCEs: $SCEs, CPEs: $CPEs, LFEs: $LFEs, DepC: $cupDEP, IndepC: $cupIND";
  } else {
    if ($ChannelConfig!=-1) { $line = $line.", ChannelCfg: $ChannelConfig"; }
    else { $line = $line.", "; }
  }

  foreach $x (@errors) {$line = $line." $x"};

  if ( $arg_outputmode =~ /line/ ) {
    print "$line\n";
  } elsif ( $arg_outputmode =~ /compare/ ) {
    #### ---- COMPARE mode output
    local $basename = "";
    local $basename_long = "";
    if ( $arg_filename ) { $basename = $arg_filename }
    elsif ( $arg_logfile ) { $basename = $arg_logfile }
    else {
      die "$line\ncan not determine base filename, can not compare";
    }

    $basename_long = $basename;
    $basename =~ s@.*/@@;
    $basename =~ s/^(?!er_)/_/;
    $basename =~ s/^([^_]*_[^_]*)_.*/$1/;
    $basename =~ s/^_//;

    #print "looking for $basename\n";

    open(NFOFILE, "< $arg_comparefile") or die "$line\ncan not open info file $arg_comparefile";

    local $x;
    local $found = 0;
    while ($x = <NFOFILE>) {
      local $y = $x;
      local $match = 0;
      if ($y =~ /^ *$basename/i ) {
        $y =~ s/^ *$basename *: *//i;
        $y =~ s/\n*$//;
        $match = 1;
      }

      if ( ($match==1)&&($y =~ s/^([0123456789,]*)://) ) {
        local $pattern = $1;
        $pattern =~ s/,/|/g;
        if ( $sampling_rate =~ /(?:$pattern)/ ) {
          $match = 1;
        } else {
          $match = 0;
        }
      }

      if ( $match==1 ) {
        $found = 1;
        $pattern = $y;
        $pattern =~ s/^  *//;
        $pattern =~ s/  */  */g;
        if ( $line =~ /$pattern$/i ) {
          print "successfully found: $basename_long ($line)\n";
          last;
        } else {
          $y =~ s/  */ /g;
          print "found an ERROR for: $basename_long\n";
          print "should be: $y\n";
          print "  but was: $line\n";
        }
      }
    }
    close(NFOFILE);

    if ($found==0) {
      print "nothing known for : $basename_long ($line)\n";
    }
  }

} else {
  #### ---- LONG mode output

  local $newline = "";
  if ( $track_total != -1) {
    print "$track_total tracks ";
    $newline = "\n";
  }
  if (length($AOTs) != 0) {
    print "AOT: $AOTs ";
    $newline = "\n";
  }

  if ($PCE_num!=0) {
    print "$newline";
    print "Channels: SCEs: $SCEs, CPEs: $CPEs, LFEs: $LFEs, DepC: $cupDEP, IndepC: $cupIND\n";
  } elsif ($ChannelConfig!=-1) {
    print "ChannelCfg: $ChannelConfig\n";
  }

  if ( @toolsused != 0 ) {
    print "TOOLS: @toolsused\n";
  }
  if ( @specials != 0 ) {
    print "SPECIAL: @specials\n";
  }
  if ( @errors != 0) {
    print "ERRORS: @errors\n";
  }
}
